@@ -441,8 +441,14 @@ gtk_source_buffer_constructor (GType                  type,
 		if (!strcmp ("tag-table", construct_param [i].pspec->name) &&
 		    g_value_get_object (construct_param [i].value) == NULL)
 		{
+#if (GLIB_MINOR_VERSION <= 2)
 			g_value_set_object_take_ownership (construct_param [i].value,
 							   gtk_source_tag_table_new ());
+#else
+			g_value_take_object (construct_param [i].value,
+					     gtk_source_tag_table_new ());
+#endif
+
 			break;
 		}
 	}
@@ -1404,7 +1410,9 @@ gtk_source_buffer_set_bracket_match_style (GtkSourceBuffer         *source_buffe
 		g_value_set_boxed (&background, &style->background);
 	else
 		g_value_set_boxed (&background, NULL);
-		
+	
+	g_object_set_property (G_OBJECT (tag), "background_gdk", &background);
+	
 	g_object_set (G_OBJECT (tag), 
 		      "style", style->italic ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL,
 		      "weight", style->bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL,
@@ -1620,7 +1628,7 @@ get_syntax_start (GtkSourceBuffer      *source_buffer,
 		tag = list->data;
 		
 		if (gtk_source_regex_match (tag->reg_start, text,
-					    pos, match->endpos))
+					    pos, match->endindex))
 			return tag;
 
 		list = g_list_next (list);
@@ -2102,7 +2110,22 @@ update_syntax_regions (GtkSourceBuffer *source_buffer,
 	    start_offset >= source_buffer->priv->worker_last_offset) {
 		/* update saved table offsets which potentially
 		 * contain the offset */
-		adjust_table_offsets (source_buffer->priv->old_syntax_regions, 0, delta);
+		region = bsearch_offset (source_buffer->priv->old_syntax_regions, start_offset);
+		if (region > 0)
+		{
+			/* Changes to the uncontrolled regions. We can't possibly
+			   know if some of the syntax regions changed, so we
+			   invalidate the saved information */
+			if (source_buffer->priv->old_syntax_regions) {
+				g_array_free (source_buffer->priv->old_syntax_regions, TRUE);
+				source_buffer->priv->old_syntax_regions = NULL;
+			}
+		}
+		else
+		{
+			adjust_table_offsets (source_buffer->priv->old_syntax_regions,
+					      region, delta);
+		}
 		return;
 	}
 	
